from datetime import datetime
import ConfigParser   
import zipfile
import time
import os.path
import re
import marshal
import shutil
import logging, logging.config
import fnmatch
from qpid.messaging import *
import qpid.datatypes

logging.config.fileConfig('log.ini')

logger = logging.getLogger('su.artix.scaleloadservice')
 
def sendMessage(message,broker):                
    data = marshal.dumps(message)   
    addressReplyTo = "%s" % qpid.datatypes.uuid4()                    
    connection = Connection(broker)
    try:
        connection.open()
        session = connection.session()
        sender = session.sender('%s; {create: always}' % (address))
        cont = {'data': data, 'host':host, 'port':port, 'statusfile':statusfile}
        request = Message(durable=True, content=cont, id=hex(hash(datetime.now())),reply_to=addressReplyTo)                        
       
        sender.send(request)
        logger.info(' Message was sent ' )
        
        session.acknowledge()
    except MessagingError,m:
        print m, MessagingError
        result = False
    finally:
        connection.close()  
if __name__ == '__main__':

    broker="localhost:5672"
    address = "artix.scales"    
    configFile = "/linuxcash/cash/scaleloadservice/scaleload.ini"
    logger.info(' Start ')
   
    c = ConfigParser.ConfigParser()
    c.read(configFile)
    section = c.sections()
    scales = {}

    for sect in section:
    	scale = {}
    	scale["flagfile"] = c.get(sect,'flagfile')
    	scale["mask"] = c.get(sect,'mask')
        scale["path"] = c.get(sect,'path')
        scale["localfile"] = c.get(sect,'localfile')
        scale["statusfile"] = c.get(sect,'statusfile')
        scale["arcfile"] = c.get(sect,'arcfile')
        scale["host"] = c.get(sect,'host')
        scale["port"] = c.get(sect,'port')
        scale["encoding"] = c.get(sect,'encoding')
        scale["loadWithZeroPrice"] = c.getboolean(sect,'loadWithZeroPrice')
        timeout  = float(c.get(sect,'timeout') )
    	scales[sect] = scale


    while True:       
        for scale in scales:
            if os.path.exists(scales[scale]["flagfile"]):
                logger.info(' Found flag %s'  % scales[scale]["flagfile"] )
                mask = scales[scale]["mask"]
                path = scales[scale]["path"]
                fileList = []
                localfile = scales[scale]["localfile"]
                statusfile = scales[scale]["statusfile"]
                arcfile = scales[scale]["arcfile"]
                host = scales[scale]["host"]
                port = scales[scale]["port"]    
                encoding =  scales[scale]["encoding"]    
                loadWithZeroPrice = scales[scale]["loadWithZeroPrice"]
                for file in os.listdir(path):
                    if fnmatch.fnmatch(file.lower(), mask):                                                      
                        fileList.append(path + file)     
                if not fileList:             
                    os.remove(scales[scale]["flagfile"])
                    continue     
                for datafile in fileList:
                    logger.info(' Found data file %s'  % datafile )                
                    head,tail = os.path.split(datafile)
                    shutil.move(datafile, localfile + tail)
                    f=open(localfile + tail, 'rb')
                    message = []
                    lineNumber = 0
                    for linedata in f:                       
                        lineNumber += 1
                        line = unicode(linedata,encoding)                       
                        if line.strip() == "":
                            continue
                        if (line[0] != 'A' and line[0] != 'D' and line[0] != '$'):
                           continue
                        if (line[0] == 'D'):
                            if (len(re.sub("\s+","",line)) != 9):
                                logger.info(' Line # %s : data length is incorrect ' % lineNumber)
                                continue
                							
                            deleteline = {}
                            deleteline["method"] = "delete"
                            deleteline["data"] = line[1:]
                            message.append(deleteline)
                        elif line[0:6] == '$CLEAR':
                            clearline = {}
                            clearline["method"] = "clear"
                            message.append(clearline)
                        else:		
                            if len(line) != 135: 
                                logger.info(' Line # %s : data length is incorrect ' % lineNumber)
                                continue		
                            addline = {}
                            addline["method"] = "add"
                            addline["barcode"] = line[1:13]
                            addline["plu"] = line[13:28]
                            addline["measure"] = line[36]
                            addline["base"] = line[37]
                            addline["price"] = float(line[38:46]) * 100
                            if (int(addline["price"]) == 0 and not loadWithZeroPrice):  
                                if(lineNumber % 200 == 0 ):  
                                    sendMessage(message,broker) 
                                    message = []                        
                                continue
                            addline["expiration"] = line[46:49]
                            addline["name"] = []
                            name = line[53:132].strip()                                                        

                            if (len(name) > 24) :
                                pos = name[0:24].rfind(' ')
                                if (pos == -1):
                                    pos = 24
                                addline["name"].append((7,name[0:pos]),)                                
                                name = name[pos:].strip()[0:56]                                
                            length = len(name)
                            font = 7
                            if (length >= 25 and length <= 36):
                                font = 4
                            if (length >=   37 and length <= 46):
                                font = 3
                            if (length >= 47):
                                font = 1
                            addline["name"].append((font, name),)                       
                            message.append(addline)                         
                        if(lineNumber % 200 == 0 ):  
                            sendMessage(message,broker) 
                            message = []
                    sendMessage(message,broker) 
                    gz=zipfile.ZipFile (arcfile, 'w', 8)  
                    gz.write(localfile + tail, None, 8)
                    gz.close()      
                os.remove(scales[scale]["flagfile"])               
        time.sleep(timeout)




							    